import numpy as np
from pyfftw.interfaces.scipy_fftpack import fft, ifft, fft2, ifft2, fftfreq, fftshift

class PTP2D:
    def __init__(self, field, lambdas, xw, yw):
        self.field = np.asarray(field)
        self.lambdas = lambdas
        self.xw = xw
        self.yw = yw

    def getField(self):
        return self.field

    def getRange(self):
        return [self.xw, self.yw]

    def getWavelength(self):
        return self.lambdas

    def getFFTField(self):
        A = fft2(self.field)
        dx = self.xw/(self.field.shape[0] - 1)
        dy = self.yw/(self.field.shape[1] - 1)
        u = fftfreq(self.field.shape[0], dx)
        v = fftfreq(self.field.shape[1], dy)
        u = fftshift(u)
        v = fftshift(v)
        A = fftshift(A)
        return A, u, v

    def setField(self, field):
        self.field = np.asarray(field)

    def setRange(self, xw, yw):
        self.xw = xw
        self.yw = yw

    def setWavelength(self, lambdas):
        self.lambdas = lambdas

    def aspw(self, zf):
        A = fft2(self.field)
        dx = self.xw/(self.field.shape[0] - 1)
        dy = self.yw/(self.field.shape[1] - 1)
        u = fftfreq(self.field.shape[0], dx)
        v = fftfreq(self.field.shape[1], dy)
        U, V = np.meshgrid(u, v)
        U = np.transpose(U)
        V = np.transpose(V)
        p = np.sqrt((1 / self.lambdas) ** 2 - U ** 2 - V ** 2 + 0j)
        G = np.exp(1j * 2 * np.pi * zf * p.real)
        H = A * G
        field_f = ifft2(H)
        return field_f

    def lp_filter(self):
        # Filter out the wavevector that larger than 1/lambda
        A = fft2(self.field)
        dx = self.xw / (self.field.shape[0] - 1)
        dy = self.yw / (self.field.shape[1] - 1)
        u = fftfreq(self.field.shape[0], dx)
        v = fftfreq(self.field.shape[1], dy)
        U, V = np.meshgrid(u, v)
        U = np.transpose(U)
        V = np.transpose(V)
        lpf = ((1 / self.lambdas) ** 2 - U ** 2 - V ** 2) > 0
        H = A * lpf
        field_f = ifft2(H)
        self.setField(field_f)

class PTP1D:
    def __init__(self, field, lambdas, xw):
        self.field = np.asarray(field)
        self.lambdas = lambdas
        self.xw = xw

    def getField(self):
        return self.field

    def getRange(self):
        return [self.xw]

    def getWavelength(self):
        return self.lambdas

    def setField(self, field):
        self.field = np.asarray(field)

    def setRange(self, xw):
        self.xw = xw

    def setWavelength(self, lambdas):
        self.lambdas = lambdas

    def aspw(self, zf):
        A = fft(self.field)
        dx = self.xw/(self.field.shape[0]-1)
        u = fftfreq(self.field.shape[0], dx)
        p = np.sqrt((1 / self.lambdas) ** 2 - u ** 2 + 0j)
        G = np.exp(1j * 2 * np.pi * zf * p.real)
        H = A * G
        field_f = ifft(H)
        return field_f

    def lp_filter(self, zf):
        A = fft(self.field)
        dx = self.xw/(self.field.shape[0]-1)
        u = fftfreq(self.field.shape[0], dx)
        lpf = ((1 / self.lambdas) ** 2 - u ** 2 + 0j) > 0
        H = A * lpf
        field_f = ifft(H)
        self.setField(field_f)
        